@@ -30,6 +30,7 @@ from piquasso.api.program import Program
 from piquasso.api.state import State
 from piquasso.api.computer import Computer
 from piquasso.api.simulator import Simulator
+from piquasso.api.utils import as_code
 
 from piquasso._backends.sampling import SamplingState, SamplingSimulator
 
@@ -70,6 +71,7 @@ from .instructions.gates import (
     ControlledZ,
     Interferometer,
     Graph,
+    CubicPhase,
 )
 
 from .instructions.measurements import (
@@ -100,6 +102,7 @@ __all__ = [
     "State",
     "Computer",
     "Simulator",
+    "as_code",
     # Simulators
     "GaussianSimulator",
     "SamplingSimulator",
@@ -133,6 +136,7 @@ __all__ = [
     "Squeezing2",
     "Kerr",
     "CrossKerr",
+    "CubicPhase",
     "ControlledX",
     "ControlledZ",
     "Interferometer",
@@ -150,4 +154,4 @@ __all__ = [
     "Loss",
 ]
 
-__version__ = "0.6.0"
+__version__ = "0.8.1"
