@@ -23,8 +23,6 @@ from .state import FockState
 from piquasso.api.instruction import Instruction
 from piquasso.api.result import Result
 
-from piquasso._math.indices import get_operator_index
-
 
 def vacuum(state: FockState, instruction: Instruction, shots: int) -> Result:
     state.reset()
@@ -35,13 +33,12 @@ def vacuum(state: FockState, instruction: Instruction, shots: int) -> Result:
 def passive_linear(state: FockState, instruction: Instruction, shots: int) -> Result:
     operator: np.ndarray = instruction._all_params["passive_block"]
 
-    index = get_operator_index(instruction.modes)
-
-    embedded_operator = np.identity(state._space.d, dtype=complex)
-
-    embedded_operator[index] = operator
-
-    fock_operator = state._space.get_passive_fock_operator(embedded_operator)
+    fock_operator = state._space.get_passive_fock_operator(
+        operator,
+        modes=instruction.modes,
+        d=state._space.d,
+        permanent_function=state._config.permanent_function,
+    )
 
     state._density_matrix = (
         fock_operator @ state._density_matrix @ fock_operator.conjugate().transpose()
@@ -139,16 +136,42 @@ def annihilate(state: FockState, instruction: Instruction, shots: int) -> Result
 def kerr(state: FockState, instruction: Instruction, shots: int) -> Result:
     mode = instruction.modes[0]
     xi = instruction._all_params["xi"]
+    for x in xi:
+        for index, (basis, dual_basis) in state._space.operator_basis:
+            number = basis[mode]
+            dual_number = dual_basis[mode]
 
-    for index, (basis, dual_basis) in state._space.operator_basis:
-        number = basis[mode]
-        dual_number = dual_basis[mode]
+            coefficient = np.exp(
+                1j
+                * x
+                * (number * (2 * number + 1) - dual_number * (2 * dual_number + 1))
+            )
 
-        coefficient = np.exp(
-            1j * xi * (number * (2 * number + 1) - dual_number * (2 * dual_number + 1))
+            state._density_matrix[index] *= coefficient
+
+    return Result(state=state)
+
+
+def cubic_phase(state: FockState, instruction: Instruction, shots: int) -> Result:
+    gamma = instruction._all_params["gamma"]
+    hbar = state._config.hbar
+
+    for index, mode in enumerate(instruction.modes):
+        operator = state._space.get_single_mode_cubic_phase_operator(
+            gamma=gamma[index], hbar=hbar
+        )
+        embedded_operator = state._space.embed_matrix(
+            operator,
+            modes=(mode,),
+            auxiliary_modes=state._get_auxiliary_modes(instruction.modes),
+        )
+        state._density_matrix = (
+            embedded_operator
+            @ state._density_matrix
+            @ embedded_operator.conjugate().transpose()
         )
 
-        state._density_matrix[index] *= coefficient
+        state.normalize()
 
     return Result(state=state)
 
@@ -229,11 +252,9 @@ def squeezing(state: FockState, instruction: Instruction, shots: int) -> Result:
 def linear(state: FockState, instruction: Instruction, shots: int) -> Result:
     operator = state._space.get_linear_fock_operator(
         modes=instruction.modes,
-        cache_size=state._config.cache_size,
-        auxiliary_modes=state._get_auxiliary_modes(instruction.modes),
         passive_block=instruction._all_params["passive_block"],
         active_block=instruction._all_params["active_block"],
-        displacement=instruction._all_params["displacement_vector"],
+        permanent_function=state._config.permanent_function,
     )
 
     state._density_matrix = (
