@@ -25,6 +25,7 @@ from .calculations import (
     density_matrix_instruction,
     kerr,
     cross_kerr,
+    cubic_phase,
     particle_number_measurement,
     vacuum,
     create,
@@ -60,6 +61,7 @@ class FockSimulator(Simulator):
         :class:`~piquasso.instructions.gates.Fourier`,
         :class:`~piquasso.instructions.gates.Kerr`,
         :class:`~piquasso.instructions.gates.CrossKerr`,
+        :class:`~piquasso.instructions.gates.CubicPhase`,
         :class:`~piquasso.instructions.gates.GaussianTransform`,
         :class:`~piquasso.instructions.gates.Squeezing`,
         :class:`~piquasso.instructions.gates.QuadraticPhase`,
@@ -86,6 +88,7 @@ class FockSimulator(Simulator):
         gates.Fourier: passive_linear,
         gates.Kerr: kerr,
         gates.CrossKerr: cross_kerr,
+        gates.CubicPhase: cubic_phase,
         gates.Squeezing: squeezing,
         gates.QuadraticPhase: linear,
         gates.Displacement: displacement,
