@@ -20,6 +20,7 @@ from .calculations import (
     passive_linear,
     kerr,
     cross_kerr,
+    cubic_phase,
     squeezing,
     displacement,
     linear,
@@ -61,6 +62,7 @@ class PureFockSimulator(Simulator):
         :class:`~piquasso.instructions.gates.Fourier`,
         :class:`~piquasso.instructions.gates.Kerr`,
         :class:`~piquasso.instructions.gates.CrossKerr`,
+        :class:`~piquasso.instructions.gates.CubicPhse`,
         :class:`~piquasso.instructions.gates.GaussianTransform`,
         :class:`~piquasso.instructions.gates.Squeezing`,
         :class:`~piquasso.instructions.gates.QuadraticPhase`,
@@ -89,6 +91,7 @@ class PureFockSimulator(Simulator):
         gates.Fourier: passive_linear,
         gates.Kerr: kerr,
         gates.CrossKerr: cross_kerr,
+        gates.CubicPhase: cubic_phase,
         gates.Squeezing: squeezing,
         gates.QuadraticPhase: linear,
         gates.Displacement: displacement,
