@@ -16,6 +16,7 @@
 from typing import Tuple, List
 
 import numpy as np
+from scipy.linalg import sqrtm
 
 from piquasso.api.config import Config
 from piquasso.api.errors import InvalidState, InvalidParameter, PiquassoException
@@ -26,7 +27,7 @@ from piquasso._math.linalg import (
     is_symmetric,
     is_positive_semidefinite,
 )
-from piquasso._math.symplectic import symplectic_form
+from piquasso._math.symplectic import symplectic_form, xp_symplectic_form
 from piquasso._math.combinatorics import get_occupation_numbers
 from piquasso._math.transformations import from_xxpp_to_xpxp_transformation_matrix
 
@@ -71,7 +72,7 @@ class GaussianState(State):
         C: np.ndarray,
         config: Config,
     ) -> "GaussianState":
-        obj = cls(d=len(m))
+        obj = cls(d=len(m), config=config)
 
         obj._m = m
         obj._G = G
@@ -606,6 +607,74 @@ class GaussianState(State):
             + (0.5 * mean @ covariance @ mean)
         )
 
+    def fidelity(self, state: "GaussianState") -> float:
+        r"""Calculates the state fidelity between two quantum states.
+
+        The state fidelity :math:`F` between two density matrices
+        :math:`\rho_1, \rho_2` is given by:
+
+        .. math::
+            \operatorname{F}(\rho_1, \rho_2) = \operatorname{Tr}(\sqrt{\sqrt{\rho_1}
+                \rho_2\sqrt{\rho_1}})^2
+
+        A gaussian state can be represented by its Covariance matrix and the vector of
+        Means. Hence, the above equation can be rewritten as:
+
+        .. math::
+            \operatorname{F} = \operatorname{F_0}(V_1, V_2) \exp(
+                -\frac{1}{2} \delta_u^T(V_1 + V_2)^{-1}\delta_u^T)
+
+        where :math:`V` is the :attr:`xpxp_covariance_matrix` of the gaussian state,
+        :math:`\delta_u` is the difference between mean vectors of the two gaussian
+        states represented by :attr:`xpxp_mean_vector`, and :math:`F_0` is given by:
+
+        .. math::
+            \operatorname{F_0} = \sqrt{\det{[2(\sqrt{I +
+                \frac{(V_{aux}\Omega)^-2}{4}} + I)V_{aux}]} \det{[(V_1 + V_2)^{-1}]}}
+
+        where :math:`V_{aux}` is given by
+
+        .. math::
+            \Omega^T (V_1 + V_2)^{-1} (\frac{\Omega}{4} V_2 \Omega V_1)
+
+        and :math:`\Omega` is a symplectic matrix of shape :math:`2*d \times 2*d`.
+        For more details please check:
+        https://journals.aps.org/prl/abstract/10.1103/PhysRevLett.115.260501.
+
+        Args:
+            state: A gaussian state
+                :class:`~piquasso._backends.gaussian.state.GaussianState` that can be
+                used to calculate the fidelity aganist it.
+
+        Returns:
+            float: The calculated fidelity.
+        """
+        mean_1, cov_1 = (
+            self.xpxp_mean_vector,
+            self.xpxp_covariance_matrix / (2 * self._config.hbar),
+        )
+        mean_2, cov_2 = (
+            state.xpxp_mean_vector,
+            state.xpxp_covariance_matrix / (2 * state._config.hbar),
+        )
+
+        W = xp_symplectic_form(self.d)
+        ident = np.identity(self.d * 2, dtype=complex)
+
+        sum_cov_inv = np.linalg.inv(cov_1 + cov_2)
+        V_aux = W.T @ sum_cov_inv @ (0.25 * W + cov_2 @ W @ cov_1)
+        delta_mu = (mean_1 - mean_2) / np.sqrt(self._config.hbar)
+
+        f1 = np.exp(-0.5 * delta_mu @ sum_cov_inv @ delta_mu)
+        f_total = (
+            2
+            * (sqrtm(ident + 0.25 * np.linalg.inv(V_aux @ W @ V_aux @ W)) + ident)
+            @ V_aux
+        )
+        f_total = np.sqrt(np.linalg.det(f_total) * np.linalg.det(sum_cov_inv))
+
+        return float((f_total * f1).real)
+
     def quadratic_polynomial_expectation(
         self, A: np.ndarray, b: np.ndarray, c: float = 0.0, phi: float = 0.0
     ) -> float:
@@ -664,6 +733,7 @@ class GaussianState(State):
         calculation = DensityMatrixCalculation(
             complex_displacement=self.complex_displacement,
             complex_covariance=self.complex_covariance,
+            loop_hafnian_function=self._config.loop_hafnian_function,
         )
 
         return calculation.get_density_matrix(
@@ -730,6 +800,7 @@ class GaussianState(State):
         calculation = DensityMatrixCalculation(
             self.complex_displacement,
             self.complex_covariance,
+            loop_hafnian_function=self._config.loop_hafnian_function,
         )
 
         return calculation.get_density_matrix_element(
@@ -742,6 +813,7 @@ class GaussianState(State):
         calculation = DensityMatrixCalculation(
             complex_displacement=self.complex_displacement,
             complex_covariance=self.complex_covariance,
+            loop_hafnian_function=self._config.loop_hafnian_function,
         )
 
         return calculation.get_particle_number_detection_probabilities(
